@@ -1,16 +1,18 @@
 """
 Model module
 """
-from typing import Callable, Dict, List, Tuple, Union, Optional
+from typing import Callable, Dict, List, Optional, Tuple, Union
 
 import numpy as np
+import pandas as pd
+from msca.linalg.matrix import Matrix
 from numpy import ndarray
 from scipy.linalg import block_diag
+from scipy.sparse import csc_matrix
 
-from regmod.data import Data
+from regmod.optimizer import scipy_optimize
 from regmod.parameter import Parameter
 from regmod.utils import sizes_to_slices
-from regmod.optimizer import scipy_optimize
 
 
 class Model:
@@ -18,8 +20,12 @@ class Model:
 
     Parameters
     ----------
-    data : Data
-        Data object.
+    obs
+        Column name for the observation
+    weights
+        Column name for the weight of each observation
+    data
+        Data frame that contains all information
     params : Optional[List[Parameter]], optional
         A list of parameters. Default to None.
     param_specs : Optional[Dict[str, Dict]], optional
@@ -32,98 +38,15 @@ class Model:
     ValueError
         Raised when data object is empty.
 
-    Attributes
-    ----------
-    data : Data
-        Data object.
-    params : List[Parameter]
-        A list of parameters.
-    sizes : List[int]
-        A list of sizes for each parameter.
-    indices : List[slice]
-        A list of indices for each parameter vector.
-    size : int
-        Sum of sizes from all parameters.
-    num_params : int
-        Number of parameters.
-    mat : np.ndarray
-        Design matrix of the problem.
-    uvec : np.ndarray
-        Direct Uniform prior array.
-    gvec : np.ndarray
-        Direct Gaussian prior array.
-    linear_umat : np.ndarray
-        Linear Uniform prior design matrix.
-    linear_gmat : np.ndarray
-        Linear Gaussian prior design matrix.
-    linear_uvec : np.ndarray
-        Linear Uniform prior array.
-    linear_gvec : np.ndarray
-        Linear Gaussian prior array.
-    opt_result : scipy.optimize.OptimizeResult
-        Optmization result. Initialized as None.
-    opt_coefs : np.ndarray
-        Optimal coefficients.
-    opt_vcov : np.ndarray
-        Optimal variance covariance matrix.
-
-    Methods
-    -------
-    get_mat()
-        Get the design matrices.
-    get_uvec()
-        Get the direct Uniform prior array.
-    get_gvec()
-        Get the direct Gaussian prior array.
-    get_linear_uvec()
-        Get the linear Uniform prior array.
-    get_linear_gvec()
-        Get the linear Gaussian prior array.
-    get_linear_umat()
-        Get the linear Uniform prior design matrix.
-    get_linear_gmat()
-        Get the linear Gaussian prior design matrix.
-    split_coefs(coefs)
-        Split coefficients into pieces for each parameter.
-    get_params(coefs)
-        Get the parameters.
-    get_dparams(coefs)
-        Get the derivative of the parameters.
-    get_d2params(coefs)
-        Get the second order derivative of the parameters.
-    nll(params)
-        Negative log likelihood.
-    dnll(params)
-        Derivative of negative the log likelihood.
-    d2nll(params)
-        Second order derivative of the negative log likelihood.
-    get_ui(params, bounds)
-        Get uncertainty interval, used for the trimming algorithm.
-    detect_outliers(coefs, bounds)
-        Detect outliers.
-    objective_from_gprior(coefs)
-        Objective function from the Gaussian priors.
-    gradient_from_gprior(coefs)
-        Gradient function from the Gaussian priors.
-    hessian_from_gprior()
-        Hessian function from the Gaussian priors.
-    objective(coefs)
-        Objective function.
-    gradient(coefs)
-        Gradient function.
-    hessian(coefs)
-        Hessian function.
-    jacobian2(coefs)
-        Jacobian function.
-    fit(optimizer, **optimizer_options)
-        Fit function.
     """
 
     param_names: Tuple[str] = None
     default_param_specs: Dict[str, Dict] = None
 
     def __init__(self,
-                 data: Data,
+                 y: str,
+                 weights: str = "weights",
+                 df: Optional[pd.DataFrame] = None,
                  params: Optional[List[Parameter]] = None,
                  param_specs: Optional[Dict[str, Dict]] = None):
         if params is None and param_specs is None:
@@ -141,29 +64,44 @@ class Model:
                                      **{**self.default_param_specs[param_name],
                                         **param_specs[param_name]})
                            for param_name in self.param_names]
-
-        self.data = data
-        if self.data.is_empty():
-            raise ValueError("Please attach dataframe before creating model.")
-        for param in self.params:
-            param.check_data(self.data)
+        self._y = y
+        self._weights = weights
+        self.df = df
+        self.y = None
+        self.weights = None
+        self.trim_weights = None
+        if self.df is not None:
+            self.attach_df(self.df)
 
         self.sizes = [param.size for param in self.params]
         self.indices = sizes_to_slices(self.sizes)
         self.size = sum(self.sizes)
         self.num_params = len(self.params)
 
+        # optimization result placeholder
+        self.opt_result = None
+        self._opt_coefs = None
+        self._opt_vcov = None
+
+    def attach_df(self, df: pd.DataFrame):
+        self.df = df
+        for param in self.params:
+            param.check_data(df)
+
         self.mat = self.get_mat()
+        self.use_hessian = not any(isinstance(m, csc_matrix) for m in self.mat)
         self.uvec = self.get_uvec()
         self.gvec = self.get_gvec()
         self.linear_uvec = self.get_linear_uvec()
         self.linear_gvec = self.get_linear_gvec()
         self.linear_umat = self.get_linear_umat()
         self.linear_gmat = self.get_linear_gmat()
-
-        # optimization result placeholder
-        self.opt_result = None
-        self._opt_coefs = None
+        self.trim_weights = np.ones(df.shape[0])
+        self.y = self.df[self._y].to_numpy()
+        if self._weights not in self.df:
+            self.weights = np.ones(self.df.shape[0])
+        else:
+            self.weights = self.df[self._weights].to_numpy()
 
     @property
     def opt_coefs(self) -> Union[None, ndarray]:
@@ -178,10 +116,31 @@ class Model:
 
     @property
     def opt_vcov(self) -> Union[None, ndarray]:
-        if self.opt_coefs is None:
-            return None
-        inv_hessian = np.linalg.pinv(self.hessian(self.opt_coefs))
-        jacobian2 = self.jacobian2(self.opt_coefs)
+        return self._opt_vcov
+
+    @opt_vcov.setter
+    def opt_vcov(self, vcov: ndarray):
+        vcov = np.asarray(vcov)
+        self._opt_vcov = vcov
+
+    def get_vcov(self, coefs: ndarray) -> ndarray:
+        hessian = self.hessian(coefs)
+        if isinstance(hessian, Matrix):
+            hessian = hessian.to_numpy()
+        eig_vals, eig_vecs = np.linalg.eig(hessian)
+        if np.isclose(eig_vals, 0.0).any():
+            raise ValueError("singular Hessian matrix, please add priors or "
+                             "reduce number of variables")
+        inv_hessian = (eig_vecs / eig_vals).dot(eig_vecs.T)
+
+        jacobian2 = self.jacobian2(coefs)
+        if isinstance(jacobian2, Matrix):
+            jacobian2 = jacobian2.to_numpy()
+        eig_vals = np.linalg.eigvals(jacobian2)
+        if np.isclose(eig_vals, 0.0).any():
+            raise ValueError("singular Jacobian matrix, please add priors or "
+                             "reduce number of variables")
+
         vcov = inv_hessian.dot(jacobian2)
         vcov = inv_hessian.dot(vcov.T)
         return vcov
@@ -194,7 +153,7 @@ class Model:
         List[ndarray]
             The design matrices.
         """
-        return [param.get_mat(self.data) for param in self.params]
+        return [param.get_mat(self.df) for param in self.params]
 
     def get_uvec(self) -> ndarray:
         """Get the direct Uniform prior array.
@@ -286,7 +245,7 @@ class Model:
             The parameters.
         """
         coefs = self.split_coefs(coefs)
-        return [param.get_param(coefs[i], self.data, mat=self.mat[i])
+        return [param.get_param(coefs[i], self.df, mat=self.mat[i])
                 for i, param in enumerate(self.params)]
 
     def get_dparams(self, coefs: ndarray) -> List[ndarray]:
@@ -303,7 +262,7 @@ class Model:
             The derivative of the parameters.
         """
         coefs = self.split_coefs(coefs)
-        return [param.get_dparam(coefs[i], self.data, mat=self.mat[i])
+        return [param.get_dparam(coefs[i], self.df, mat=self.mat[i])
                 for i, param in enumerate(self.params)]
 
     def get_d2params(self, coefs: ndarray) -> List[ndarray]:
@@ -320,7 +279,7 @@ class Model:
             The second order derivative of the parameters.
         """
         coefs = self.split_coefs(coefs)
-        return [param.get_d2param(coefs[i], self.data, mat=self.mat[i])
+        return [param.get_d2param(coefs[i], self.df, mat=self.mat[i])
                 for i, param in enumerate(self.params)]
 
     def nll(self, params: List[ndarray]) -> ndarray:
@@ -407,7 +366,8 @@ class Model:
         """
         params = self.get_params(coefs)
         ui = self.get_ui(params, bounds)
-        return (self.data.obs < ui[0]) | (self.data.obs > ui[1])
+        obs = self.df[self.y].to_numpy()
+        return (obs < ui[0]) | (obs > ui[1])
 
     def objective_from_gprior(self, coefs: ndarray) -> float:
         """Objective function from the Gaussian priors.
@@ -458,6 +418,12 @@ class Model:
             hess += (self.linear_gmat.T/self.linear_gvec[1]**2).dot(self.linear_gmat)
         return hess
 
+    def get_nll_terms(self, coefs: ndarray) -> ndarray:
+        params = self.get_params(coefs)
+        nll_terms = self.nll(params)
+        nll_terms = self.weights*nll_terms
+        return nll_terms
+
     def objective(self, coefs: ndarray) -> float:
         """Objective function.
 
@@ -471,10 +437,8 @@ class Model:
         float
             Objective value.
         """
-        params = self.get_params(coefs)
-        obj_params = self.nll(params)
-        weights = self.data.weights*self.data.trim_weights
-        return weights.dot(obj_params) + \
+        nll_terms = self.get_nll_terms(coefs)
+        return self.trim_weights.dot(nll_terms) + \
             self.objective_from_gprior(coefs)
 
     def gradient(self, coefs: ndarray) -> ndarray:
@@ -493,7 +457,7 @@ class Model:
         params = self.get_params(coefs)
         dparams = self.get_dparams(coefs)
         grad_params = self.dnll(params)
-        weights = self.data.weights*self.data.trim_weights
+        weights = self.weights*self.trim_weights
         return np.hstack([
             dparams[i].T.dot(weights*grad_params[i])
             for i in range(self.num_params)
@@ -517,7 +481,7 @@ class Model:
         d2params = self.get_d2params(coefs)
         grad_params = self.dnll(params)
         hess_params = self.d2nll(params)
-        weights = self.data.weights*self.data.trim_weights
+        weights = self.weights*self.trim_weights
         hess = [
             [(dparams[i].T*(weights*hess_params[i][j])).dot(dparams[j])
              for j in range(self.num_params)]
@@ -543,7 +507,7 @@ class Model:
         params = self.get_params(coefs)
         dparams = self.get_dparams(coefs)
         grad_params = self.dnll(params)
-        weights = self.data.weights*self.data.trim_weights
+        weights = self.weights*self.trim_weights
         jacobian = np.vstack([
             dparams[i].T*(weights*grad_params[i])
             for i in range(self.num_params)
@@ -561,4 +525,39 @@ class Model:
         optimizer : Callable, optional
             Model solver, by default scipy_optimize.
         """
+        if self.size == 0:
+            self.opt_coefs = np.empty((0,))
+            self.opt_vcov = np.empty((0, 0))
+            self.opt_result = "no parameter to fit"
+            return
         optimizer(self, **optimizer_options)
+
+    def predict(self, df: Optional[pd.DataFrame] = None) -> pd.DataFrame:
+        """Predict the parameters.
+
+        Parameters
+        ----------
+        df : pd.DataFrame, optional
+            Data Frame with prediction data. If it is None, using the training
+            data.
+
+        Returns
+        -------
+        pd.DataFrame
+            Data frame with predicted parameters.
+        """
+        if df is None:
+            df = self.df
+        df = df.copy()
+
+        coefs = self.split_coefs(self.opt_coefs)
+        for i, param_name in enumerate(self.param_names):
+            df[param_name] = self.params[i].get_param(coefs[i], df)
+
+        return df
+
+    def __repr__(self) -> str:
+        return (f"{type(self).__name__}("
+                f"bnum_y={self.df.shape[0]}, "
+                f"num_params={self.num_params}, "
+                f"size={self.size})")
