@@ -2,12 +2,12 @@
 Parameter module
 """
 from dataclasses import dataclass, field
-from typing import List, Tuple, Union
+from typing import Optional, Union
 
 import numpy as np
+import pandas as pd
 from scipy.linalg import block_diag
 
-from regmod.data import Data
 from regmod.function import SmoothFunction, fun_dict
 from regmod.prior import LinearGaussianPrior, LinearUniformPrior
 from regmod.variable import SplineVariable, Variable
@@ -21,72 +21,29 @@ class Parameter:
 
     Parameters
     ----------
-    name : str
+    name
         Name of the parameter.
-    variables : List[Variable]
+    variables
         A list of variables that parametrized the parameter.
-    inv_link : Union[str, SmoothFunction]
+    inv_link
         Inverse link funcion to link variables to parameter.
-    use_offset : bool, optional
-        If `True` when build parameter, it will use the offset in the data
-        object. Default to `False`.
-    linear_gpriors : List[LinearGaussianPrior], optional
+    offset
+        If `offset=None` parameter will not use offset, when it is a string
+        it will look for the corresponding column in the data frame as the
+        offset.
+    linear_gpriors
         A list of linear Gaussian priors. Default to an empty list.
-    linear_upriors : List[LinearUniformPrior], optional
+    linear_upriors
         A list of linear Uniform priors. Default to an empty list.
 
-    Attributes
-    ----------
-    name : str
-        Name of the parameter.
-    variables : List[Variable]
-        A list of variables that parametrized the parameter.
-    inv_link : Union[str, SmoothFunction]
-        Inverse link funcion to link variables to parameter.
-    use_offset : bool
-        If `True` when build parameter, it will use the offset in the data
-        object.
-    linear_gpriors : List[LinearGaussianPrior]
-        A list of linear Gaussian priors.
-    linear_upriors : List[LinearUniformPrior]
-        A list of linear Uniform priors.
-    size : int
-        Size of the parameter. It equals to sum of the sizes of the variables.
-
-    Methods
-    -------
-    check_data(data)
-        Attach data to all variables.
-    get_mat(data)
-        Get the design matrix.
-    get_uvec()
-        Get the direct Uniform prior.
-    get_gvec()
-        Get the direct Gaussian prior.
-    get_linear_uvec()
-        Get the linear Uniform prior vector.
-    get_linear_gvec()
-        Get the linear Gaussian prior vector.
-    get_linear_umat()
-        Get the linear Uniform prior matrix.
-    get_linear_gmat()
-        Get the linear Gaussian prior matrix.
-    get_lin_param(coefs, data, mat=None, return_mat=False)
-        Get the parameter before apply the link function.
-    get_param(coefs, data, mat=None)
-        Get the parameter.
-    get_dparam(coefs, data, mat=None)
-        Get the derivative of the parameter.
-    get_d2param(coefs, data, mat=None)
-        Get the second order derivative of the parameter.
     """
 
     name: str
-    variables: List[Variable] = field(repr=False)
-    inv_link: Union[str, SmoothFunction] = field(repr=False)
-    use_offset: bool = field(default=False, repr=False)
-    linear_gpriors: List[LinearGaussianPrior] = field(default_factory=list, repr=False)
-    linear_upriors: List[LinearUniformPrior] = field(default_factory=list, repr=False)
+    variables: list[Variable] = field(default_factory=list, repr=False)
+    inv_link: Union[str, SmoothFunction] = field(default="identity", repr=False)
+    offset: Optional[str] = field(default=None, repr=False)
+    linear_gpriors: list[LinearGaussianPrior] = field(default_factory=list, repr=False)
+    linear_upriors: list[LinearUniformPrior] = field(default_factory=list, repr=False)
 
     def __post_init__(self):
         if isinstance(self.inv_link, str):
@@ -103,37 +60,44 @@ class Parameter:
             "linear_upriors size not match."
         assert all([isinstance(var, Variable) for var in self.variables]), \
             "variables has to be a list of Variable."
+        if len(self.variables) == 0 and self.offset is None:
+            raise ValueError("Please provide at least one variable when "
+                             "parameter does not have an offset")
 
     @property
     def size(self) -> int:
         """Size of the parameter."""
         return sum([var.size for var in self.variables])
 
-    def check_data(self, data: Data):
+    def check_data(self, df: pd.DataFrame):
         """Attach data to all variables.
 
         Parameters
         ----------
-        data : Data
-            Data object.
+        df
+            Data frame that contains all covariates.
+
         """
         for var in self.variables:
-            var.check_data(data)
+            var.check_data(df)
 
-    def get_mat(self, data: Data) -> np.ndarray:
+    def get_mat(self, df: pd.DataFrame) -> np.ndarray:
         """Get the design matrix.
 
         Parameters
         ----------
-        data : Data
-            Data object.
+        df
+            Data frame contains all covariates.
 
         Returns
         -------
         np.ndarray
             The design matrix.
+
         """
-        return np.hstack([var.get_mat(data) for var in self.variables])
+        if len(self.variables) == 0:
+            return np.empty(shape=(df.shape[0], 0))
+        return np.hstack([var.get_mat(df) for var in self.variables])
 
     def get_uvec(self) -> np.ndarray:
         """Get the direct Uniform prior.
@@ -143,6 +107,8 @@ class Parameter:
         np.ndarray
             Uniform prior information array.
         """
+        if len(self.variables) == 0:
+            return np.empty(shape=(2, 0))
         return np.hstack([var.get_uvec() for var in self.variables])
 
     def get_gvec(self) -> np.ndarray:
@@ -153,6 +119,8 @@ class Parameter:
         np.ndarray
             Gaussian prior information array.
         """
+        if len(self.variables) == 0:
+            return np.empty(shape=(2, 0))
         return np.hstack([var.get_gvec() for var in self.variables])
 
     def get_linear_uvec(self) -> np.ndarray:
@@ -163,6 +131,8 @@ class Parameter:
         np.ndarray
             Uniform prior information array.
         """
+        if len(self.variables) == 0:
+            return np.empty(shape=(2, 0))
         uvec = np.hstack([
             var.get_linear_uvec() if isinstance(var, SplineVariable) else np.empty((2, 0))
             for var in self.variables
@@ -180,6 +150,8 @@ class Parameter:
         np.ndarray
             Gaussian prior information array.
         """
+        if len(self.variables) == 0:
+            return np.empty(shape=(2, 0))
         gvec = np.hstack([
             var.get_linear_gvec() if isinstance(var, SplineVariable) else np.empty((2, 0))
             for var in self.variables
@@ -197,6 +169,8 @@ class Parameter:
         np.ndarray
             Uniform prior design matrix.
         """
+        if len(self.variables) == 0:
+            return np.empty(shape=(0, 0))
         umat = block_diag(*[
             var.get_linear_umat() if isinstance(var, SplineVariable) else np.empty((0, 1))
             for var in self.variables
@@ -213,6 +187,8 @@ class Parameter:
         np.ndarray
             Gaussian prior design matrix.
         """
+        if len(self.variables) == 0:
+            return np.empty(shape=(0, 0))
         gmat = block_diag(*[
             var.get_linear_gmat() if isinstance(var, SplineVariable) else np.empty((0, 1))
             for var in self.variables
@@ -223,17 +199,17 @@ class Parameter:
 
     def get_lin_param(self,
                       coefs: np.ndarray,
-                      data: Data,
+                      df: pd.DataFrame,
                       mat: np.ndarray = None,
-                      return_mat: bool = False) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
+                      return_mat: bool = False) -> Union[np.ndarray, tuple[np.ndarray, np.ndarray]]:
         """Get the parameter before apply the link function.
 
         Parameters
         ----------
         coefs : np.ndarray
             Coefficients for the design matrix.
-        data : Data
-            Data object.
+        df
+            Data frame that contains all the covariates.
         mat : np.ndarray, optional
             Alternative design matrix, by default None.
         return_mat : bool, optional
@@ -241,22 +217,25 @@ class Parameter:
 
         Returns
         -------
-        Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]
+        Union[np.ndarray, tuple[np.ndarray, np.ndarray]]
             Linear parameter vector, or when `return_mat=True` also returns the
             design matrix.
+
         """
+        if len(self.variables) == 0:
+            return df[self.offset].to_numpy()
         if mat is None:
-            mat = self.get_mat(data)
+            mat = self.get_mat(df)
         lin_param = mat.dot(coefs)
-        if self.use_offset:
-            lin_param += data.offset
+        if self.offset is not None:
+            lin_param += df[self.offset].to_numpy()
         if return_mat:
             return lin_param, mat
         return lin_param
 
     def get_param(self,
                   coefs: np.ndarray,
-                  data: Data,
+                  df: pd.DataFrame,
                   mat: np.ndarray = None) -> np.ndarray:
         """Get the parameter.
 
@@ -264,8 +243,8 @@ class Parameter:
         ----------
         coefs : np.ndarray
             Coefficients for the design matrix.
-        data : Data
-            Data object.
+        df
+            Data frame that contains all the covariates.
         mat : np.ndarray, optional
             Alternative design matrix, by default None.
 
@@ -274,12 +253,12 @@ class Parameter:
         np.ndarray
             Returns the parameter.
         """
-        lin_param = self.get_lin_param(coefs, data, mat)
+        lin_param = self.get_lin_param(coefs, df, mat)
         return self.inv_link.fun(lin_param)
 
     def get_dparam(self,
                    coefs: np.ndarray,
-                   data: Data,
+                   df: pd.DataFrame,
                    mat: np.ndarray = None) -> np.ndarray:
         """Get the derivative of the parameter.
 
@@ -287,8 +266,8 @@ class Parameter:
         ----------
         coefs : np.ndarray
             Coefficients for the design matrix.
-        data : Data
-            Data object.
+        df
+            Data frame that contains all the covariates.
         mat : np.ndarray, optional
             Alternative design matrix, by default None.
 
@@ -297,12 +276,14 @@ class Parameter:
         np.ndarray
             Returns the derivative of the parameter.
         """
-        lin_param, mat = self.get_lin_param(coefs, data, mat, return_mat=True)
+        if len(self.variables) == 0:
+            return np.empty((df.shape[0], 0))
+        lin_param, mat = self.get_lin_param(coefs, df, mat, return_mat=True)
         return self.inv_link.dfun(lin_param)[:, None]*mat
 
     def get_d2param(self,
                     coefs: np.ndarray,
-                    data: Data,
+                    df: pd.DataFrame,
                     mat: np.ndarray = None) -> np.ndarray:
         """Get the second order derivative of the parameter.
 
@@ -310,8 +291,8 @@ class Parameter:
         ----------
         coefs : np.ndarray
             Coefficients for the design matrix.
-        data : Data
-            Data object.
+        df
+            Data frame that contains all the covariates.
         mat : np.ndarray, optional
             Alternative design matrix, by default None.
 
@@ -320,5 +301,7 @@ class Parameter:
         np.ndarray
             Returns the second order derivative of the parameter.
         """
-        lin_param, mat = self.get_lin_param(coefs, data, mat, return_mat=True)
+        if len(self.variables) == 0:
+            return np.empty((df.shape[0], 0, 0))
+        lin_param, mat = self.get_lin_param(coefs, df, mat, return_mat=True)
         return self.inv_link.d2fun(lin_param)[:, None, None]*(mat[..., None]*mat[:, None, :])
